//Building a Better Build Null Object
//Author : Steven Pettit 01 Jan 2015
//Adds a null item to a scene, parents it to a selected item
//And sets the item shapes, colors, text, etc.
//Based on a complaint on the Lightwave Forums

//TODO :  Make spacing of the interface elements a little more
//Consistent;  may need to use some fast UDF's in the interface
//Code to get things just right.

@warnings
@script generic
@name Add Null 2.0

//Globals for UI Helper functions
var GRIDSIZE = 24;
var OFFSET = 8;

generic{
	var pipon;
    var selectedItem;
    var MyParent;
	var tmpi_i;
	var fmt;
	var nullflag = true;

	//Trap version, itemshapedata() requires 11.6 or better
    if (hostVersion() < 11.6) error("This script works with version 11.6 or higher!");

	fmt = itemshapedata();
	
	//Let's see if we have anything selected
	pipon = Scene().generalopts[3];
    selectedItem = Scene().firstSelect();
    if(!selectedItem){
		nullflag = false;
	} else {
	    MyParent = selectedItem.id;
		nullflag = true;
	}
	
	//Interface Code Here!
	itemname = "Null"; if(nullflag) itemname = selectedItem.name + "_child";
	nullitemshape = false;
    nullshape = 1;
    nullaxis = 1;
    nullfilled = false;
    nullxray = false;
    nulllabel = "";
    nulljustify = 2;
    nulldraw = nil;
    nullSelColorFlag = false;
    nullselcolor = <0,0,0>;
    nullUnSelColorFlag = false;
    nullUnSelColor = <0,0,0>;
    nullTextColorFlag = false;
    nullTextColor = <0,0,0>;
    nullscale = 1.0;
    nullOpacity = 1.0;
    nullParent = false; if(nullflag) nullParent = true;
	nullmult = false;
	nullquant = 1;
	nullchain = false;
	
	lh = 20;
    reqbegin("Advanced Build Null Object, Ver. 1.0");
    reqsize(grido(24)+8,grido(17)+8);
	
	//Give the null a name
    c1 = ctlstring("Item Name",itemname);
    ctlposition(c1,grido(5),grido(0),grid(14),lh);

	s1 = ctlsep(grido(0),grid(24));
    ctlposition(s1,grido(0),grido(1)+8,grid(24),24);
   
	//Major Options
	c17 = ctlcheckbox("Parent to Selected",nullParent);
    ctlposition(c17,grido(8),grido(2),grid(8),lh);
	
	s5 = ctlsep(grido(0),grid(24)); ctlposition(s5,grido(0),grido(3)+8,grid(24),24);
	
	//Multiple Nulls and Chain Options
	c19 = ctlcheckbox("Multiple Nulls", nullmult); ctlposition(c19,grido(2),grido(4),grid(5),lh);
	c20 = ctlnumber("Quantity",nullquant);ctlposition(c20,grido(8),grido(4),grid(5),lh);ctlactive(c19,"additemshapeactive",c20);
	c21 = ctlcheckbox("Make Null Chain",nullchain);ctlposition(c21,grido(15),grido(4),grid(5),lh);ctlactive(c19,"additemshapeactive",c21);
	
	s4 = ctlsep(grido(0),grid(24));
    ctlposition(s4,grido(0),grido(5)+8,grid(24),24);
	
	//Item Shape Options
	c18 = ctlcheckbox("Add Item Shape",nullitemshape);
    ctlposition(c18,grido(9),grido(6),grid(6),lh);	
	
	c22 = ctlchoice("Shape", nullshape,@"Standard","Box","Ball","Pyramid","Diamond","Tetrahedron","Ring","Grid","None"@);
	ctlposition(c22,grido(0),grido(7),grid(23),lh);
	ctlactive(c18,"additemshapeactive",c22);
	
	//c2 = ctlpopup("Shape",nullshape,@"Standard","Box","Ball","Pyramid","Diamond","Tetrahedron","Ring","Grid","None"@);
    //ctlposition(c2,136,168,120,lh);
	//ctlactive(c18,"additemshapeactive",c2);
	
    c15 = ctldistance("Scale",nullscale);
    ctlposition(c15,grido(0),grido(8),grid(4),lh);
	ctlactive(c18,"additemshapeactive",c15);
	
	c3 = ctlchoice("Axis",nullaxis,@"X","Y","Z"@);
    ctlposition(c3,grido(5),grido(8),grid(4),lh);
	ctlactive(c18,"additemshapeactive",c3);

    c4 = ctlcheckbox("Filled",nullfilled);
    ctlposition(c4,grido(12),grido(8),grid(4),lh);
	ctlactive(c18,"additemshapeactive",c4);

    c5 = ctlcheckbox("X Ray",nullxray);
    ctlposition(c5,grido(19),grido(8),grid(4),lh);
	ctlactive(c18,"additemshapeactive",c5);

    s2 = ctlsep(grido(0),grid(24));
    ctlposition(s2,grido(0),grido(9)+8,grid(24),24);

	//Text and Labelling
    c6 = ctlstring("Label",nulllabel); ctlposition(c6,grido(0),grido(10),grid(9),lh);
    c7 = ctlpopup("Justification",nulljustify,@"Left","Center","Right"@); ctlposition(c7,grido(10),grido(10),grid(6),lh);
    c8 = ctlallitems("Draw To",nulldraw); ctlposition(c8,grido(17),grido(10),grid(7),lh);
	s3 = ctlsep(grido(0),grid(24)); ctlposition(s3,grido(0),grido(11)+8,grid(24),24);
	
	//Colors
	yo = 48;
    c9 = ctlcheckbox("Selected Color",nullSelColorFlag);
    ctlposition(c9,grido(5),grido(12),grid(5),lh);	

    c10 = ctlcolor(" ",nullselcolor);
    ctlposition(c10,grido(12),grido(12),grid(5),lh);
	
    c11 = ctlcheckbox("Unselected Color",nullUnSelColorFlag);
    ctlposition(c11,grido(5),grido(13),grid(5),lh);
	
    c12 = ctlcolor(" ",nullUnSelColor);
    ctlposition(c12,grido(12),grido(13),grid(5),lh);
	
    c13 = ctlcheckbox("Text Color",nullTextColorFlag);
    ctlposition(c13,grido(5),grido(14),grid(5),lh);
	
    c14 = ctlcolor(" ",nullTextColor);
    ctlposition(c14,grido(12),grido(14),grid(5),lh);
	    
    c16 = ctlpercent("Opacity",nullOpacity);
    ctlposition(c16,grido(11),grido(15),grid(5),lh);
		
    return if !reqpost();

    itemname = getvalue(c1);
    nullshape = getvalue(c22);
    nullaxis = getvalue(c3);
    nullfilled = getvalue(c4);
    nullxray = getvalue(c5);
    nulllabel = getvalue(c6);
    nulljustify = getvalue(c7);
    nulldraw = getvalue(c8);
    nullSelColorFlag = getvalue(c9);
    nullSelColor = getvalue(c10);
    nullUnSelColorFlag = getvalue(c11);
    nullUnSelColor = getvalue(c12);
    nullTextColorFlag = getvalue(c13);
    nullTextColor = getvalue(c14);
    nullscale = getvalue(c15);
    nullOpacity = getvalue(c16);
    nullParent = getvalue(c17);
	nullitemshape = getvalue(c18);
	nullmult = getvalue(c19);
	nullquant = getvalue(c20);
	nullchain = getvalue(c21);

    reqend();
	
	//LW_ItemShape uses zero based indices.
	nullaxis -= 1;
	nullshape -= 1;
	nulljustify -= 1;
	//It Also uses floating point color values
	nullSelColor /= <255,255,255>;
	nullUnSelColor /= <255,255,255>;
	nullTextColor /= <255,255,255>;
	//And I don't want to take chances with this being out of 0.0 to 1.0
	nullOpacity = min(max(nullOpacity,0),1);

	//Let's make some nulls!
	//debug();
	if (!nullmult){	
		tmp_i = makenull (itemname, nullParent, MyParent, pipon, nullitemshape,
					nullshape, nullaxis, nullscale, nullfilled, nullxray,
					nulllabel, nulljustify, nulldraw,
					nullSelColorFlag, nullSelColor,
					nullUnSelColorFlag, nullUnSelColor,
					nullTextColorFlag, nullTextColor,
					nullOpacity);
		
	} else {
		for(i=1;i<=nullquant;i++)
		{
			tmp_name = itemname + format("_$1",i);
			tmp_i = makenull (tmp_name, nullParent, MyParent, pipon, nullitemshape,
					nullshape, nullaxis, nullscale, nullfilled, nullxray,
					nulllabel, nulljustify, nulldraw,
					nullSelColorFlag, nullSelColor,
					nullUnSelColorFlag, nullUnSelColor,
					nullTextColorFlag, nullTextColor,
					nullOpacity);		
			if(nullchain) MyParent = tmp_i.id;
			if(!nullParent && nullchain) nullParent = true;
		}
	}
}

makenull : w_name, w_parF, w_par, w_pip, w_aisF, w_shape, w_axis, w_size, w_fill, w_xray, w_label, w_just, w_draw, w_selF, w_selC, w_unsF, w_unsC, w_txtF, w_txtC, w_opac
{
	//Refactored the code into a stand alone UDF.  AddNull is a lScript command, so it's makenull.
	if (w_pip) ParentInPlace();
	AddNull(w_name);
	tmp_i = Scene().firstSelect();
	SelectItem(tmp_i.id);
	if(w_parF) ParentItem(w_par);
	if (w_pip) ParentInPlace();
	Position(0,0,0);
	Rotation(0,0,0);
	if(w_aisF){
		ApplyServer("CustomObjHandler","LW_ItemShape");
		additemshape(tmp_i, w_shape, w_axis, w_size, w_fill, w_xray, w_label, w_just, w_draw, w_selF, w_selC, w_unsF, w_unsC, w_txtF, w_txtC, w_opac);
	}
	return(tmp_i);
}


additemshape: w_item, w_shape, w_axis, w_size, w_fill, w_xray, w_label, w_just, w_draw, w_selF, w_selC, w_unsF, w_unsC, w_txtF, w_txtC, w_opac
{
	var l_fmt = nil;
	var l_data = nil;
	var l_info = nil;
	//Get Item Shape Data Encoding Format
	l_fmt = itemshapedata();
	
	//Check to make sure that it does have the LW_Itemshape plugin applied
	SelectItem(w_item.id);
	tv = w_item.server(SERVER_CUSTOMOBJ_H);
	if (tv[1] == "LW_ItemShape") cr = true;
	if(cr){
		//And, if it does, let's grab what it's set to right now...
		comringattach("ItemShapeComRing", "shapecomring");
		l_data = comringencode(l_fmt,
								0, 
								w_item.id, 0, 
								0.0, 0.0, 0.0,
								0, 0, 0, 0, 0, 
								0.0, 0.0, 0.0, 
								0.0, 0.0, 0.0, 
								0.0, 0.0, 0.0, 
								0.0, 0.0, 0.0,
								"", 
								"",
								"", 
								0, 
								"", 
								"",
								0 );
		comringmsg("ItemShapeComRing", 100, l_data);
		//Decode that information and place it in an array...
		l_info = comringdecode(l_fmt, l_data);	
		//Let's populate that array with our data;
		l_info[2] = w_item.id;
		l_info[4] = 0.0;
		l_info[5] = w_opac;	
		l_info[6] = w_size;	
		l_info[7] = w_axis;
		l_info[8] = w_fill; 
		l_info[9] = w_xray;
		l_info[10] = w_shape;
		l_info[21] = 0.0; l_info[22] = 0.0; l_info[23] = 0.0;
		l_info[11] = 4 * w_txtF + 2 * w_unsF + w_selF;
		l_info[27] = 0;
		l_info[30] = w_just;
		
		if (w_selF){
			l_info[12] = w_selC.x; 
			l_info[13] = w_selC.y; 
			l_info[14] = w_selC.z;
		}
		if (w_unsF){
			l_info[15] = w_unsC.x;
			l_info[16] = w_unsC.y;
			l_info[17] = w_unsC.z;
		}
		if (w_txtF){
			l_info[18] = w_txtC.x;
			l_info[19] = w_txtC.y;
			l_info[20] = w_txtC.z; 
		}
		if(w_label != ""){ 
			l_info[24] = w_label; 
			l_info[25] = w_label;
		}
		if(w_draw){
			l_info[3] = w_draw.id; 
			l_info[29] = w_draw.name;
		}
		
		l_data = comringencode(l_fmt,
                                    l_info[1],
                                    l_info[2], l_info[3],
                                    l_info[4],l_info[5],l_info[6],
                                    l_info[7], l_info[8],l_info[9],l_info[10],l_info[11],
                                    l_info[12],l_info[13],l_info[14],
                                    l_info[15],l_info[16],l_info[17],
                                    l_info[18],l_info[19],l_info[20],
                                    l_info[21],l_info[22],l_info[23],
                                    l_info[24],
                                    l_info[25],
                                    l_info[26],
                                    l_info[27],
                                    l_info[28],
                                    l_info[29],
                                    l_info[30]);
		comringmsg("ItemShapeComRing", 200, l_data);			
		comringdetach("ItemShapeComRing");			
	}
}
// A do-nothing function for the comring to hook up to
shapecomring: code, data
{
}

//Attempt to use ctlactive to hide or activate other controls...
additemshapeactive : value
{
	//Since control is a checkbox, this should do.
	return(value);
}

grido : w_coord
{
	var retv = grid(w_coord);
	retv += OFFSET;
	return (retv);
}

grid : w_coord
{
	var retv = 0;
	retv = GRIDSIZE * w_coord;
	return (retv);
}
