//Building a Better Build Null Object
//Author : Steven Pettit 21 Aug 2016
//Adds a null item to a scene, parents it to a selected item
//And sets the item shapes, colors, text, etc.
//19Aug08 :  Added Control Pairs, Text Formatting
//Based on a complaint on the Lightwave Forums

@warnings
@script generic
@name Add Null 3.0

//Globals for UI Helper function
var GRIDSIZE = 22;
var SKEW = floor(GRIDSIZE / 2);
var OFFSET = floor(SKEW / 2);
var nullflag = true;	
var LW_CONTROL = 0;
var LW_MAIN = 1;
var LW_SEPARATOR = 2;

//Work around - I don't want to pass any more arguments to makenull...
var offsetnullf = false;
var offsetnullv = <0,0,0>;

generic{
//-->
	var pipon;
    var selectedItem;
    var MyParent;
	var tmpi_i;
	
	var a[4]; 
//<--	
//-->
	//Trap version, itemshapedata() requires 11.6 or better
    if (hostVersion() < 11.6) error("This script works with version 11.6 or higher!");
	
	//Let's see if we have anything selected
	pipon = Scene().generalopts[3];
    selectedItem = Scene().firstSelect();
    if(!selectedItem){
		nullflag = false;
	} else {
	    MyParent = selectedItem.id;
		nullflag = true;
	}
//<--		
//-->
	//Interface Code Here!
	itemname = "Null"; if(nullflag) itemname = selectedItem.name + "_child";
	nullitemshape = false;
    nullshape = 1;
    nullaxis = 1;
    nullfilled = false;
    nullxray = false;
    nulllabel = "";
    nulljustify = 2;
    nulldraw = nil;
    nullSelColorFlag = false;
    nullselcolor = <0,0,0>;
    nullUnSelColorFlag = false;
    nullUnSelColor = <0,0,0>;
    nullTextColorFlag = false;
    nullTextColor = <0,0,0>;
    nullscale = 1.0;
    nullOpacity = 1.0;
    nullParent = false; if(nullflag) nullParent = true;
	nullmult = false;
	nullquant = 2;
	nullchain = false;
	nullpair = false;
	nulloffset = false;
	nullvector = <0,0,1>;
//<--
//Open a menu -->	
    reqbegin("Advanced Build Null Object, Ver. 2.0");
    a = gbox2(LW_MAIN,0,0,24,19);reqsize(a[3],a[4]);
	//<--
	//Give the null a name -->
    c1 = ctlstring("Item Name",itemname);
    a = gbox2(LW_CONTROL,5,0,14,1); ctlposition(c1,a[1],a[2],a[3],a[4]);
	a = gbox2(LW_SEPARATOR,0,1,24,1); s1 = ctlsep(a[1],a[3]); ctlposition(s1,a[1],a[2],a[3],a[4]);
	c50 = ctlcheckbox("dummy", nullflag); ctlposition(c50,0,0,0,0);
	//<--
	//Major Options -->
	c17 = ctlcheckbox("Parent to Selected",nullParent);
	a = gbox2(LW_CONTROL, 2,2,8,1); ctlposition(c17,a[1],a[2],a[3],a[4]); ctlactive(c50,"getnullflag",c17);
	c51 = ctlcheckbox("Make Control Pair",nullpair);
	a = gbox2(LW_CONTROL,14,2,8,1); ctlposition(c51,a[1],a[2],a[3],a[4]);
	a = gbox2(LW_SEPARATOR,0,3,24,1); s5 = ctlsep(a[1],a[3]); ctlposition(s5,a[1],a[2],a[3],a[4]);
	//<--
	//Multiple Nulls and Chain Options -->
	a = gbox2(LW_CONTROL, 2,4,5,1); c19 = ctlcheckbox("Multiple Nulls", nullmult); ctlposition(c19,a[1],a[2],a[3],a[4]);
	a = gbox2(LW_CONTROL,8,4,5,1); c20 = ctlinteger("Quantity",nullquant);ctlposition(c20,a[1],a[2],a[3],a[4]);ctlactive(c19,"additemshapeactive",c20);
	a = gbox2(LW_CONTROL,15,4,5,1); c21 = ctlcheckbox("Make Null Chain",nullchain);ctlposition(c21,a[1],a[2],a[3],a[4]);ctlactive(c19,"additemshapeactive",c21);
	a = gbox2(LW_CONTROL,15,5,3,1); c52 = ctlcheckbox("Offset",nulloffset);ctlposition(c52,a[1],a[2],a[3],a[4]);
	a = gbox2(LW_CONTROL,19,5,4,1); c53 = ctlvector("",nullvector);ctlposition(c53,a[1],a[2],a[3],a[4]);
	ctlactive(c52,"additemshapeactive",c53);
	a = gbox2(LW_SEPARATOR,0,6,24,1); s4 = ctlsep(a[1],a[3]); ctlposition(s4,a[1],a[2],a[3],a[4]);
	//<--
	//Item Shape Options -->
	a = gbox2(LW_CONTROL,9,7,6,1); c18 = ctlcheckbox("Add Item Shape",nullitemshape); ctlposition(c18,a[1],a[2],a[3],a[4]);	
	
	c22 = ctlchoice("Shape", nullshape,@"Standard","Box","Ball","Pyramid","Diamond","Tetrahedron","Ring","Grid","None"@);
	a = gbox2(LW_CONTROL,0,8,23,1); ctlposition(c22,a[1],a[2],a[3],a[4]); ctlactive(c18,"additemshapeactive",c22);
	
    a = gbox2(LW_CONTROL,0,9,4,1); c15 = ctldistance("Scale",nullscale); ctlposition(c15,a[1],a[2],a[3],a[4]);
	ctlactive(c18,"additemshapeactive",c15);
	
	a = gbox2(LW_CONTROL,5,9,4,1); c3 = ctlchoice("Axis",nullaxis,@"X","Y","Z"@); ctlposition(c3,a[1],a[2],a[3],a[4]);
	ctlactive(c18,"additemshapeactive",c3);

    a = gbox2(LW_CONTROL,12,9,4,1); c4 = ctlcheckbox("Filled",nullfilled);
    ctlposition(c4,a[1],a[2],a[3],a[4]);
	ctlactive(c18,"additemshapeactive",c4);

    a = gbox2(LW_CONTROL,19,9,4,1);c5 = ctlcheckbox("X Ray",nullxray);
    ctlposition(c5,a[1],a[2],a[3],a[4]);
	ctlactive(c18,"additemshapeactive",c5);

    a=gbox2(LW_SEPARATOR,0,10,24,1); s2 = ctlsep(a[1],a[3]); ctlposition(s2,a[1],a[2],a[3],a[4]);
	//<--
	//Text and Labelling -->
    a = gbox2(LW_CONTROL,0,11,9,1); c6 = ctlstring("Label",nulllabel); ctlposition(c6,a[1],a[2],a[3],a[4]);
    a = gbox2(LW_CONTROL,10,11,6,1); c7 = ctlpopup("Justification",nulljustify,@"Left","Center","Right"@); ctlposition(c7,a[1],a[2],a[3],a[4]);
    a = gbox2(LW_CONTROL,17,11,7,1); c8 = ctlallitems("Draw To",nulldraw); ctlposition(c8,a[1],a[2],a[3],a[4]);
	a = gbox2(LW_SEPARATOR,0,12,24,1); s3 = ctlsep(a[1],a[3]); ctlposition(s3,a[1],a[2],a[3],a[4]);
	//<--
	//Colors -->
	
    a = gbox2(LW_CONTROL,5,13,6,1); c9 = ctlcheckbox("Selected Color",nullSelColorFlag); ctlposition(c9,a[1],a[2],a[3],a[4]);	
    a = gbox2(LW_CONTROL,12,13,6,1); c10 = ctlcolor(" ",nullselcolor); ctlposition(c10,a[1],a[2],a[3],a[4]); ctlactive(c9,"additemshapeactive",c10);
	
    a = gbox2(LW_CONTROL,5,14,6,1); c11 = ctlcheckbox("Unselected Color",nullUnSelColorFlag); ctlposition(c11,a[1],a[2],a[3],a[4]);
	a = gbox2(LW_CONTROL,12,14,6,1); c12 = ctlcolor(" ",nullUnSelColor); ctlposition(c12,a[1],a[2],a[3],a[4]); ctlactive(c11,"additemshapeactive",c12);
	
    a = gbox2(LW_CONTROL,5,15,6,1); c13 = ctlcheckbox("Text Color",nullTextColorFlag); ctlposition(c13,a[1],a[2],a[3],a[4]);
	a = gbox2(LW_CONTROL,12,15,6,1); c14 = ctlcolor(" ",nullTextColor); ctlposition(c14,a[1],a[2],a[3],a[4]); ctlactive(c13,"additemshapeactive",c14);
	    
    a= gbox2(LW_CONTROL,11,16,6,1); c16 = ctlpercent("Opacity",nullOpacity); ctlposition(c16,a[1],a[2],a[3],a[4]);
	//<--	
	//Get our Values from the requester -->
    return if !reqpost();

    itemname = getvalue(c1);
    nullshape = getvalue(c22);
    nullaxis = getvalue(c3);
    nullfilled = getvalue(c4);
    nullxray = getvalue(c5);
    nulllabel = getvalue(c6);
    nulljustify = getvalue(c7);
    nulldraw = getvalue(c8);
    nullSelColorFlag = getvalue(c9);
    nullSelColor = getvalue(c10);
    nullUnSelColorFlag = getvalue(c11);
    nullUnSelColor = getvalue(c12);
    nullTextColorFlag = getvalue(c13);
    nullTextColor = getvalue(c14);
    nullscale = getvalue(c15);
    nullOpacity = getvalue(c16);
    nullParent = getvalue(c17);
	nullitemshape = getvalue(c18);
	nullmult = getvalue(c19);
	nullquant = getvalue(c20);
	nullchain = getvalue(c21);
	nullpair = getvalue(c51);
	nulloffset = getvalue(c52);
	nullvector = getvalue(c53);
    reqend();
//<--

//Map Values and Trap Ambiguous Cases -->
	//LW_ItemShape uses zero based indices.
	nullaxis -= 1;
	nullshape -= 1;
	nulljustify -= 1;
	//It Also uses floating point color values
	nullSelColor /= <255,255,255>;
	nullUnSelColor /= <255,255,255>;
	nullTextColor /= <255,255,255>;
	//And I don't want to take chances with this being out of 0.0 to 1.0
	nullOpacity = min(max(nullOpacity,0),1);
	//Let's trap this one - No Selection, but Parent to Selection checked
	nullParent = (nullflag && nullParent);
//<--
//Let's make some nulls! -->
	if(nullpair){
		//Special Case - makes 2 nulls, with special names, ctrl parented to base 
		
		myitemname = makeitemname(itemname,"base",0);
		nullitemshape = false;
		tmp_i = makenull (myitemname, nullParent, MyParent, pipon, nullitemshape,
					nullshape, nullaxis, nullscale, nullfilled, nullxray,
					nulllabel, nulljustify, nulldraw,
					nullSelColorFlag, nullSelColor,
					nullUnSelColorFlag, nullUnSelColor,
					nullTextColorFlag, nullTextColor,
					nullOpacity);
		Scale(0.1,0.1,0.1);
		MyParent = tmp_i.id;
		nullParent = true;
		myitemname = makeitemname(itemname,"ctrl",0);
		nullitemshape = true;
		tmp_i = makenull (myitemname, nullParent, MyParent, pipon, nullitemshape,
					nullshape, nullaxis, nullscale, nullfilled, nullxray,
					nulllabel, nulljustify, nulldraw,
					nullSelColorFlag, nullSelColor,
					nullUnSelColorFlag, nullUnSelColor,
					nullTextColorFlag, nullTextColor,
					nullOpacity);
	} else {
		if (!nullmult){
			tmp_i = makenull (itemname, nullParent, MyParent, pipon, nullitemshape,
					nullshape, nullaxis, nullscale, nullfilled, nullxray,
					nulllabel, nulljustify, nulldraw,
					nullSelColorFlag, nullSelColor,
					nullUnSelColorFlag, nullUnSelColor,
					nullTextColorFlag, nullTextColor,
					nullOpacity);
		} else {
			offsetnullf = nulloffset;
			if(nulloffset) offsetnullv = nullvector;
			for(i=1;i<=nullquant;i++)
			{
				
				myitemname = makeitemname(itemname,"",i);
				tmp_i = makenull (myitemname, nullParent, MyParent, pipon, nullitemshape,
					nullshape, nullaxis, nullscale, nullfilled, nullxray,
					nulllabel, nulljustify, nulldraw,
					nullSelColorFlag, nullSelColor,
					nullUnSelColorFlag, nullUnSelColor,
					nullTextColorFlag, nullTextColor,
					nullOpacity);		
				if(nullchain) MyParent = tmp_i.id;
				if(!nullchain) offsetnullv += nullvector;
				if(!nullParent && nullchain) nullParent = true;
			}
		}
	}	
}
//<--

makeitemname: q_base, q_str, q_cnt
{
	//Provides a means to insert strings and enumeration
	//at someplace other than the end.  Uses Regular Expressions
	//To do the grunt work, and normal string functions if
	//%s and/or %n are not present...
	retv = "";
	astr = "(%s)";
	bstr = "(%n)";
	work = q_base;
	tmps = work;
	if(q_str != ""){
		if(tmps == regexp(astr)){
			rexpr = regexp(astr,q_str);
			work ~= rexpr;
		} else {
			work += "_" + q_str;
		}
	}
	tmps = work;
	if(q_cnt > 0){
		c_str = format("$1",q_cnt);
		if (c_str.size() < 2) c_str = "0" + c_str;
		if(tmps == regexp(bstr)){
			rexpr = regexp(bstr,c_str);
			work ~= rexpr;
		} else {
			work += "_" + c_str;
		}
	}
	retv = work;
	return(retv);
}

makenull : w_name, w_parF, w_par, w_pip, w_aisF, w_shape, w_axis, w_size, w_fill, w_xray, w_label, w_just, w_draw, w_selF, w_selC, w_unsF, w_unsC, w_txtF, w_txtC, w_opac
{
	//Refactored the code into a stand alone UDF.  AddNull is a lScript command, so it's makenull.
	if (w_pip) ParentInPlace();
	AddNull(w_name);
	tmp_i = Scene().firstSelect();
	SelectItem(tmp_i.id);
	if(w_parF) ParentItem(w_par);
	if (w_pip) ParentInPlace();
	if(offsetnullf){ Position(offsetnullv.x, offsetnullv.y, offsetnullv.z); } else { Position(0,0,0);}
	Rotation(0,0,0);
	if(w_aisF){
		ApplyServer("CustomObjHandler","LW_ItemShape");
		additemshape(tmp_i, w_shape, w_axis, w_size, w_fill, w_xray, w_label, w_just, w_draw, w_selF, w_selC, w_unsF, w_unsC, w_txtF, w_txtC, w_opac);
	}
	return(tmp_i);
}


additemshape: w_item, w_shape, w_axis, w_size, w_fill, w_xray, w_label, w_just, w_draw, w_selF, w_selC, w_unsF, w_unsC, w_txtF, w_txtC, w_opac
{
	var l_fmt = nil;
	var l_data = nil;
	var l_info = nil;
	//Get Item Shape Data Encoding Format
	l_fmt = itemshapedata();
	
	//Check to make sure that it does have the LW_Itemshape plugin applied
	SelectItem(w_item.id);
	tv = w_item.server(SERVER_CUSTOMOBJ_H);
	if (tv[1] == "LW_ItemShape") cr = true;
	if(cr){
		//And, if it does, let's grab what it's set to right now...
		comringattach("ItemShapeComRing", "shapecomring");
		l_data = comringencode(l_fmt,
								0, 
								w_item.id, 0, 
								0.0, 0.0, 0.0,
								0, 0, 0, 0, 0, 
								0.0, 0.0, 0.0, 
								0.0, 0.0, 0.0, 
								0.0, 0.0, 0.0, 
								0.0, 0.0, 0.0,
								"", 
								"",
								"", 
								0, 
								"", 
								"",
								0 );
		comringmsg("ItemShapeComRing", 100, l_data);
		//Decode that information and place it in an array...
		l_info = comringdecode(l_fmt, l_data);	
		//Let's populate that array with our data;
		l_info[2] = w_item.id;
		l_info[4] = 0.0;
		l_info[5] = w_opac;	
		l_info[6] = w_size;	
		l_info[7] = w_axis;
		l_info[8] = w_fill; 
		l_info[9] = w_xray;
		l_info[10] = w_shape;
		l_info[21] = 0.0; l_info[22] = 0.0; l_info[23] = 0.0;
		l_info[11] = 4 * w_txtF + 2 * w_unsF + w_selF;
		l_info[27] = 0;
		l_info[30] = w_just;
		
		if (w_selF){
			l_info[12] = w_selC.x; 
			l_info[13] = w_selC.y; 
			l_info[14] = w_selC.z;
		}
		if (w_unsF){
			l_info[15] = w_unsC.x;
			l_info[16] = w_unsC.y;
			l_info[17] = w_unsC.z;
		}
		if (w_txtF){
			l_info[18] = w_txtC.x;
			l_info[19] = w_txtC.y;
			l_info[20] = w_txtC.z; 
		}
		if(w_label != ""){ 
			l_info[24] = w_label; 
			l_info[25] = w_label;
		}
		if(w_draw){
			l_info[3] = w_draw.id; 
			l_info[29] = w_draw.name;
		}
		
		l_data = comringencode(l_fmt,
                                    l_info[1],
                                    l_info[2], l_info[3],
                                    l_info[4],l_info[5],l_info[6],
                                    l_info[7], l_info[8],l_info[9],l_info[10],l_info[11],
                                    l_info[12],l_info[13],l_info[14],
                                    l_info[15],l_info[16],l_info[17],
                                    l_info[18],l_info[19],l_info[20],
                                    l_info[21],l_info[22],l_info[23],
                                    l_info[24],
                                    l_info[25],
                                    l_info[26],
                                    l_info[27],
                                    l_info[28],
                                    l_info[29],
                                    l_info[30]);
		comringmsg("ItemShapeComRing", 200, l_data);			
		comringdetach("ItemShapeComRing");			
	}
}
// A do-nothing function for the comring to hook up to
shapecomring: code, data
{
}

//Attempt to use ctlactive to hide or activate other controls...
additemshapeactive : value
{
	//Since control is a checkbox, this should do.
	return(value);
}

getnullflag : value
{
	var retv = nullflag;
	return(retv);
}

gbox : w_x, w_y, w_h, w_w
{
	var retv[4];
	retv[1] = (w_x * GRIDSIZE) + OFFSET;
	retv[2] = (w_y * GRIDSIZE) + OFFSET;
	retv[3] = (w_h * GRIDSIZE);
	retv[4] = (w_w * GRIDSIZE);
	return(retv);
}

gbox2 : w_mode, w_x, w_y, w_h, w_w
{
	//w_mode 0 = normal, 1 = main box, 2 = Separator
	//All values are GRID coordinates
	//w_x, w_y = (x,y) coordinates of control;
	//w_h, w_w = (height, width) of control;
	
	//Return Values array
	var retv[4];
	
	//Convert from Grid coordinates to screen coordinates
	retv[1] = (w_x * GRIDSIZE) + OFFSET;
	retv[2] = (w_y * GRIDSIZE) + OFFSET;
	retv[3] = (w_h * GRIDSIZE);
	retv[4] = (w_w * GRIDSIZE);
	
	//Mode Handling
	if (w_mode == 1){
		retv[3] += SKEW;
		retv[4] += OFFSET;
	}
	if (w_mode == 2) retv[2] += SKEW;
	
	return(retv);
}